// These are duplicated in preferenceDialog.js. Change here -> change there, too.
var showOwnTagsKey = 'delicious.showOwnTags';
var showPopRecTagsKey = 'delicious.showPopRecTags';

/**
 * Called when the "post link" dialog box is created. This retrieves the
 * default values for the form parameters from the URL that specifies the
 * box itself. See for example postCurrentPage() in deliciousOverlay.js.
 */
function loadArgs()
{
    var args=window.arguments[0];

    var url = "";
    var description = "";
    var extended = "";
    var tags = "";
    var pair;

    if (args['url'] != null) url=decodeURIComponent(args['url']);
    if (args['description'] != null) description=decodeURIComponent(args['description']);
    if (args['extended'] != null) extended=decodeURIComponent(args['extended']);
    if (args['tags'] != null) tags=decodeURIComponent(args['tags']);
    if (args['globals'] == null) {
        alert("Del.icio.us extension internal error: globals missing");
        return;
    }
    globals = args['globals'];
    
    if (globals.deliciousService.getUsername().length == 0 || globals.deliciousService.getPassword().length == 0) {
        alert("Please set up an account using the Del.icio.us extension's options first.");
        window.close();
        return;
    }
   
    document.getElementById("url").value = url;
    document.getElementById("description").value = description;
    document.getElementById("extended").value = extended;
    document.getElementById("tags").value = tags;

    var wordsInExtended = extended.replace(/\s+/g, ' ').replace(/^\W+|\W+$/g, '').split(' ').length;
    document.getElementById("extended-row").setAttribute("flex", wordsInExtended/2);

    document.getElementById('extended').focus();
    
    var users = globals.userManager.getUsers();
    var defaultUser = globals.userManager.getDefaultUser().getUsername();
    var postAsPopup = document.getElementById('use-account-popup');
    for (i in users) {
        var item = document.createElement("menuitem");
        item.setAttribute('label', users[i].getAccountName());
        item.setAttribute('value', users[i].getUsername());
        var node = postAsPopup.appendChild(item);
        if (users[i].getUsername() == defaultUser) {
            document.getElementById('use-account-list').selectedItem = item;
        }
    }

    extendedModified();
    if (nsPreferences.getBoolPref(showOwnTagsKey, true) == true) {
      var tags = globals.userManager.getDefaultUser().tags;
      var errorMsg = tags.gettingTagsFailed ? "Failed to get tags from del.icio.us server" : "You have no tags defined.";
      displayTags(globals.userManager.getDefaultUser().tags.getTagNamesSorted(), "tags-your", errorMsg, "tags-your-row", g_popularTags.concat(g_recommendedTags));
	} else {
	    hideElement("tags-your-row");
	}
    if (nsPreferences.getBoolPref(showPopRecTagsKey, true) == true) {
      var text = "Loading tags from del.icio.us.";
      showStatusText("tags-recommended", text);
      showStatusText("tags-popular", text);
      loadExperimentalFields(url, description);
	} else {
      hideElement("tags-recommended-row");
      hideElement("tags-popular-row");
	}
}

function hideElement(id) {
    document.getElementById(id).hidden = true;
}

// Globals, needed to retain some info through the asynchronous loading process.
var g_url = '';
var g_username = '';
var g_password = '';
var g_description = '';
var g_recommendedTags = [];
var g_popularTags = [];
var gExperimentalLoaded = false;

// Globals from the caller. Needs to contain deliciousService and userManager objects.
var globals;

/*
 * Load del.icio.us' experimental post page, parse tags from it and populate
 * the dialog's "experimental" field with them.
 */
function loadExperimentalFields(url, description)
{
    if (gExperimentalLoaded == true) return;
    
   	g_username = globals.deliciousService.getUsername();
    g_password = globals.deliciousService.getPassword();
   	if (g_username.length == 0 || g_password.length == 0) {
   	    window.close();
   	    return;
   	}
    g_url = url;
    g_description = description;
    
	var queryURL = "http://del.icio.us/new/" + encodeURIComponent(g_username) +
		"?v=2&url=" + encodeURIComponent(g_url) +
		"&title=" + encodeURIComponent(g_description);
    globals.cookies.turnCookiesOn();
    asyncGetURL(queryURL, g_username, g_password, false, experimentalAsyncGetURLRequester);
}

//
// Callback for the asynchronous getting of user's tags.
//
var experimentalAsyncGetURLRequester = {
    // Process the asynchronous getting of the experimental fields info.
    asyncGetURLResponse: function(resp) {
        globals.cookies.restoreCookieState();
        if (resp != null) {
            gExperimentalLoaded = true;
            var tags = parseTagsFromResponse(resp, /var\s+tagRec\s*=\s*\[\s*(".*"|\s*,\s*)*\s*]/i);
            displayTags(tags, "tags-recommended", "No recommended tags found.", "tags-recommended-row", tags);
            g_recommendedTags = tags;
            tags = parseTagsFromResponse(resp, /var\s+tagPop\s*=\s*\[\s*(".*"|\s*,\s*)*\s*]/i);
            displayTags(tags, "tags-popular", "No popular tags found.", "tags-popular-row", tags);
            g_popularTags = tags;
        }
    },

    // Process error.
    asyncGetURLError: function(error) {
        globals.cookies.restoreCookieState();
        return;
    }
};

//
// Parse tags from the experimental response.
//
function parseTagsFromResponse(resp, regexp)
{
    var tagmatch = resp.match(regexp);
    var tags = [];
    if (tagmatch && tagmatch[1] != null) {
        var tagarray = tagmatch[1].replace(/"/g, '')
        tags = tagarray.split(/\s*,\s*/);
    }
    return tags;
}

//
// Display tags or give a note if none are found. Emphasize given tags.
//
function displayTags(tags, elementId, noneFoundMessage, rowId, tagsToEmphasize)
{
    if (tags.length == 0) {
        showStatusText(elementId, noneFoundMessage);
    } else {
        writeTagsToDialogElement(tags, elementId);
        styleElementTags('tags-your', tagArrayToObject(tagsToEmphasize), function(e){e.style.color = 'red;'});
        document.getElementById(rowId).setAttribute("flex", tags.length);
    }
}

//
// Indicate an error in loading the experimental fields.
//
function showExperimentalError(explanation, exception)
{
    var r = document.getElementById("tags-recommended");
    r.value = explanation + exception;
    r = document.getElementById("tags-popular");
    r.value = explanation + exception;
}    

//
// Write a status text into a tag field.
//
function showStatusText(elementId, aText)
{
    clearLabelsFromElement(elementId);
    var label = document.createElement("label");
    label.setAttribute("value", aText);
    label.setAttribute("style", "font-style: italic;");
    document.getElementById(elementId).appendChild(label);
}

//
// Clear any existing tags or text (labels) from a tag field.
//
function clearLabelsFromElement(elementId)
{
    var element = document.getElementById(elementId);
    var labels = element.getElementsByTagName('label');
    for (var i = labels.length - 1; i >= 0; --i) {
        element.removeChild(labels.item(i));
    }
}

/**
 * Store tags to a XULElement.
 */
function writeTagsToDialogElement(tags, elementId)
{
    clearLabelsFromElement(elementId);
    var element = document.getElementById(elementId);
    for (i in tags) {
        var label = document.createElement("label");
        label.setAttribute("value", tags[i]);
        label.setAttribute("onclick", "tagClicked(this.value,'"+elementId+"');");
        element.appendChild(label);
    }
}

/**
 * Set style for given tags in the given element's tag list.
 */
function styleElementTags(aElementId, aTags, aStyleSetter, aStyleClearer)
{
    var nodelist = document.getElementById(aElementId).childNodes;
    for (var j = nodelist.length - 1; j >= 0; j--) {
        var node = nodelist.item(j);
        if (node.getAttribute('value') in aTags) {
            aStyleSetter(node);
        } else if(aStyleClearer) {
            aStyleClearer(node);
        }
    }
}

/**
 * Switch an own tag on or off.
 */
function switchOwnTag(tag)
{
    var tags = stripTagString(document.getElementById("tags").value).split(' ');
    var present = false;
    for (t in tags) {
        if (tags[t] == tag) {
            tags.splice(t,1);
            present = true;
        }
    }
    if (!present) {
        tags.push(tag);
    }
    document.getElementById("tags").value = (tags.join(" ")+' ').replace(/\s+/g, ' ').replace(/^\s/, '');
}

/**
 * Tag clicked on the UI.
 */
function tagClicked(tag, elementId)
{
  if (elementId == "tags-suggestions") {
    var tags = stripTagString(document.getElementById("tags").value).split(" ");
    var prevTags = gPrevTagString.split(" ");
    var tagsChanged = false;
    for (t in tags) {
      if (prevTags[t] != tags[t]) {
        tags.splice(t, 1);
        tagsChanged = true;
        break;
      }
    }
    if (tagsChanged) {
      document.getElementById("tags").value = (tags.join(" ")+' ').replace(/\s+/g, ' ').replace(/^\s/, '');
    }
  }
  switchOwnTag(tag);
  tagsModified();
  gPrevTagString = stripTagString(document.getElementById("tags").value);
  clearLabelsFromElement("tags-suggestions");
}

/**
 * Change the account to post to a different one.
 */
function changeAccount(event)
{
    clearLabelsFromElement("tags-your");
    globals.userManager.setDefaultUser(event.target.value);
    if (nsPreferences.getBoolPref(globals.deliciousService.showOwnTagsKey, true) == true) {
      displayTags(globals.userManager.getDefaultUser().tags.getTagNamesSorted(), "tags-your", "You have no tags defined.", "tags-your-row", g_popularTags.concat(g_recommendedTags));
      var tags = tagArrayToObject(document.getElementById("tags").value.split(" "));
      styleElementTags("tags-your", tags, setTagElementSelected, setTagElementUnselected);
    }
}

/**
 * Executed when user confirms the "post link" dialog box.
 */
function accept()
{
    var url = document.getElementById("url").value;
    var description = document.getElementById("description").value;
    var extended = document.getElementById("extended").value;
    var tags = document.getElementById("tags").value;
    var date = new Date();

    globals.deliciousService.postURL(url, description, extended, tags, date.toISO8601String(), acceptAsyncGetURLRequester);
    document.documentElement.getButton("accept").disabled = true;
    document.getElementById("status-row").hidden = false;
    document.getElementById("status-text").value = "Posting data."

    tags = tags.match(/\s*(\S.*\S)\s*/);
    if (tags != null) {
        var userTags = globals.userManager.getDefaultUser().tags;
        userTags.addTags(tags[1].split(/\s+/));
        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observerService.notifyObservers(null,"ext:delicious:tagxmlForDefaultUser", userTags.asXml(true));
    }

    return false;
}

/**
 * Executed when the user cancels the post dialog.
 */
function cancelled()
{
    globals.cookies.restoreCookieState();
    return true;
}

var acceptAsyncGetURLRequester = {
    // Executed when the URL has been posted.
    asyncGetURLResponse: function(resp)
    {
        if (resp != null) {
            document.getElementById("postsDialog").cancelDialog();
            
            //TODO This is a "fix" for Firefox 1.5beta1, Thunderbird 1.5beta1, and Deer Park Alpha2. The cancelDialog doesn't work in them.
            document.getElementById("status-text").value += " Done. Click cancel to close the dialog. ";

        } else {
            this.logError();
        }
    },

    // Process error.
    asyncGetURLError: function(error) {
        this.logError();
        return;
    },

    logError: function() {
        document.documentElement.getButton("accept").disabled = false;
        var date = new Date();
        document.getElementById("status-text").value = "Posting failed ("+date.toISO8601String()+"). Click ok to retry or cancel to exit.";
    }
};

/**
 * Format a date as required to make a del.icio.us post (the &dt= parameter).
 * This uses ISO 8601 format: "CCYY-MM-DDThh:mm:ssZ"
 * See here for details: http://www.cl.cam.ac.uk/~mgk25/iso-time.html
 * Example: 1984-09-01T14:21:31Z
 */
Date.prototype.toISO8601String = function()
{
    var str;
    
    // date
    var year = this.getUTCFullYear().toString();
    var month = (this.getUTCMonth()+1).toString();
    month = month.length < 2 ? ("0" + month) :  month;
    
    var day = this.getUTCDate().toString();
    day = day.length < 2 ? ("0" + day) : day;
    
    var hours = this.getUTCHours().toString();
    hours = hours.length < 2 ? ("0" + hours) : hours;
    
    var minutes = this.getUTCMinutes().toString();
    minutes = minutes.length < 2 ? ("0" + minutes) : minutes;
    
    var seconds = this.getUTCSeconds().toString();
    seconds = seconds.length < 2 ? ("0" + seconds) : seconds;
    
    return year + "-" + month + "-" + day + "T" + hours + ":" + minutes + ":" + seconds + "Z";
}

/**
 * Parse an ISO 8601 formatted string into a Date object.
 */
Date.prototype.fromISO8601String = function(str)
{
    str = str.replace("Z","");
    var parts = str.split("T");
    var dateArr = parts[0].split("-");
    var timeArr = parts[1].split(":");
    
    this.setUTCFullYear(dateArr[0]);
    this.setUTCMonth(dateArr[1]-1);
    this.setUTCDate(dateArr[2]);
    this.setUTCHours(timeArr[0]);
    this.setUTCMinutes(timeArr[1]);
    this.setUTCSeconds(timeArr[2]);
}

/**
 * Update extended field's length info when the field is changed.
 */
function extendedModified()
{
    var length = getEncodedLength(document.getElementById("extended").value);
    document.getElementById("extended-label").value = length + " / 255";
}

/**
 * Get encoded byte count
 */
function getEncodedLength(text)
{
    var count = 0;
    for (var i=0; i<text.length; i++) {
        count += Math.ceil(encodeURIComponent(text.charAt(i)).length / 3);
    }
    return count;
}

/**
 * Add a bunch of tags.
 */
function addAllTags(tagSet)
{
    var tags = tagSet == "recommended" ? g_recommendedTags : g_popularTags;
    for (t in tags) {
      switchOwnTag(tags[t]);
    }
    tagsModified();
}

function tagArrayToObject(aArray)
{
    var tags = {};
    for (t in aArray) {
        tags[aArray[t]] = true;
    }
    return tags;
}

function setTagElementUnselected(e) {e.style.background = 'transparent;'}
function setTagElementSelected(e) {e.style.background = '#ffbbbb;'}

/**
 * Tag field modified, set coloring accordingly.
 */
function tagsModified(event)
{
    var tags = tagArrayToObject(stripTagString(document.getElementById("tags").value).split(" "));
    styleElementTags("tags-your", tags, setTagElementSelected, setTagElementUnselected);
    styleElementTags("tags-popular", tags, setTagElementSelected, setTagElementUnselected);
    styleElementTags("tags-recommended", tags, setTagElementSelected, setTagElementUnselected);
}

/**
 * Find suggestions for the current word.
 */
function findSuggestions(tags)
{
  var userTags = globals.userManager.getDefaultUser().tags.getTagNamesSortedByFrequency();
  var userTagsObj = tagArrayToObject(userTags);

  var tagString = stripTagString(document.getElementById("tags").value);
  var currentTags = tagString.split(" ");
  var prevTags = gPrevTagString.split(" ");
  var fract = "";

  for (t in currentTags) {
    if (prevTags[t] != currentTags[t] && !(currentTags[t] in userTagsObj)) {
      if (currentTags.length == prevTags.length || currentTags[t].length == 1) { // Don't suggest if tags have been split by the previous edit.
        fract = currentTags[t];
      }
      break;
    }
  }

  var suggestions = [];
  if (fract != "") {
    var allTags = userTags.concat(g_recommendedTags.concat(g_popularTags));
    var sugmapped = tagArrayToObject(document.getElementById("tags").value.split(" "));
    for (t in allTags) {
      if (fract.length < allTags[t].length && allTags[t].indexOf(fract) == 0 && !sugmapped[allTags[t]])  {
        suggestions.push(allTags[t]);
        sugmapped[allTags[t]] = true;
      }
    }
  }
  if (suggestions.length > 0) {
    writeTagsToDialogElement(suggestions, "tags-suggestions");
    setTagElementSelected(document.getElementById("tags-suggestions").getElementsByTagName('label').item(0));
  } else {
    clearLabelsFromElement("tags-suggestions");
  }
}

function stripTagString(aStr)
{
  return aStr.replace(/\s+/g, ' ').replace(/^\s|\s$/g,'');
}

var gPrevTagString = "";
/**
 * Key pressed in tag area. Check if it's a tab and complete the tag if it is.
 */
function tagsOnKeyPress(event)
{
  var tagString = stripTagString(document.getElementById("tags").value.replace(/\s+/g, ' ').replace(/^\s/,''));
  if (event.keyCode == event.DOM_VK_TAB) {
    var suggestions = document.getElementById("tags-suggestions").getElementsByTagName('label');
    if (suggestions.length > 0) {
      var suggestion = suggestions.item(0).value;
      var tags = tagString.split(" ");
      var prevTags = gPrevTagString.split(" ");
      for (t in tags) {
        if (prevTags[t] != tags[t] && tags[t].length < suggestion.length && suggestion.indexOf(tags[t]) == 0) {
          tags.splice(t, 1, suggestion);
          tagString = tags.join(" ");
          document.getElementById("tags").value = (tagString+' ').replace(/\s\s$/, ' ');
          tagsModified();
          event.preventDefault();
          break;
        }
      }
    }
  }
  if (tagString != gPrevTagString) {
    gPrevTagString = tagString;
  }
}

/**
 * Key released in tag area. Check if suggestions should be updated.
 */
function tagsOnKeyUp(event)
{
  switch (event.keyCode) {
    case event.DOM_VK_SPACE:
    case event.DOM_VK_END:
    case event.DOM_VK_HOME:
    case event.DOM_VK_LEFT:
    case event.DOM_VK_RIGHT:
      clearLabelsFromElement("tags-suggestions");
      break;
    default:
      findSuggestions(tagArrayToObject(stripTagString(document.getElementById("tags").value).split(" ")));
      break;
  }
}
