/*
* Copyright (C) 2005, Pasi P. Liimatainen
* Distributable under GNU General Public License (v2)
* See terms of license at http://www.gnu.org
*/

//
// Cookie management.
// Getting the popular and recommended tags requires logging into del.icio.us with cookies on.
//

function Cookies(aGlobals)
{
  this.enableCookieManagementKey = 'delicious.enableCookieManagement';
  this.globals = aGlobals;
  this.cookieBehavior = -1;

  /**
   * Turn cookies on, save original setting for later.
   * Return true if cookie management was allowed, false otherwise.
   */
  this.turnCookiesOn = function() {
    if (!this.globals.onThunderbird) {
      return true;
    }
    if (nsPreferences.getBoolPref(this.enableCookieManagementKey, true) == true) {
      if (this.cookieBehavior == -1) {
        this.cookieBehavior = nsPreferences.getIntPref("network.cookie.cookieBehavior", 2);
      }
      nsPreferences.setIntPref("network.cookie.cookieBehavior", 1);  // 0-Accept, 1-dontAcceptForeign, 2-dontUse, 3-p3p
      return true;
    }
    return false;
  };

  /**
   * Restore cookies to original setting.
   */
  this.restoreCookieState = function() {
    if (!this.globals.onThunderbird) {
      return;
    }
    if (this.cookieBehavior != -1) {
      nsPreferences.setIntPref("network.cookie.cookieBehavior", this.cookieBehavior);
    }
  };

  this.managementEnabled setter = function(enabled) {
    nsPreferences.setBoolPref(this.enableCookieManagementKey, enabled);
  };

  this.managementEnabled getter = function() {
    return nsPreferences.getBoolPref(this.enableCookieManagementKey, true);
  };

  this.setPermission = function(aPermission) {
    var ioservice = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var uri = ioservice.newURI("http://del.icio.us/", null, null);
    var cookiePermissionService = Components.classes["@mozilla.org/cookie/permission;1"].getService(Components.interfaces.nsICookiePermission);
    cookiePermissionService.setAccess(uri, aPermission);
  };

  this.allow = function() {
    if (this.globals.onThunderbird) {
      return;
    }
    var permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"].getService(Components.interfaces.nsIPermissionManager);
    var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
    uri.spec = "http://del.icio.us/";
    if (permissionmanager.testPermission(uri, "cookie") == Components.interfaces.nsIPermissionManager.ALLOW_ACTION) {
      return;
    }
    permissionmanager.add(uri, "cookie", Components.interfaces.nsIPermissionManager.ALLOW_ACTION);
  };

  this.disallow = function() {
    if (this.globals.onThunderbird) {
      return;
    }
    var permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"].getService(Components.interfaces.nsIPermissionManager);
    var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
    uri.spec = "http://del.icio.us/";
    if (permissionmanager.testPermission(uri, "cookie") != Components.interfaces.nsIPermissionManager.UNKNOWN_ACTION) {
      permissionmanager.remove(uri.host, "cookie");
    }
  };

  if (this.managementEnabled) {
    this.allow();
  } else {
    this.disallow();
  }
}
