function MessagingService() {
  this.mMessagingTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
  this.mMessagingTimer.initWithCallback(this, 2000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
  this.setDefaultUserRequestUsername = null;
  this.notify = function(aTimer) {
    var name = this.setDefaultUserRequestUsername;
    if (name != null) {
      this.setDefaultUserRequestUsername = null;
      globals.userManager.setDefaultUser(name);
    }
  };
  this.setDefaultUserRequest = function(aUsername) {
    this.setDefaultUserRequestUsername = aUsername;
  };
};

// This is not very nice, but don't know of a nicer way to share
// stateful objects between XUL windows than passing globals around.
function Globals()
{
    this.onThunderbird getter = function() {
      return /Thunderbird/.test(navigator.userAgent);
    }
    this.deliciousService = new DeliciousService(this);
    this.userManager = new UserManager(this.deliciousService);
    this.deliciousService.hookUserManager(this.userManager);
    this.messagingService = new MessagingService();
    this.cookies = new Cookies(this);
}
const globals = new Globals();

globals.userManager.setDefaultUser(globals.userManager.getDefaultUser().getUsername());  // Log in, preload tags, etc.

var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);

function preloadTags(index) {
  const users = globals.userManager.getUsers();
  if (index < users.length) {
    if (users[index].getUsername() == globals.userManager.getDefaultUser().getUsername()) {
      preloadTags(index+1); // Default user's tags are preloaded elsewhere, so skip them here.
      return;
    }
    users[index].tags.preloadTags(preloadTags, index+1, users[index].getUsername(), users[index].getPassword());
  }
}

preloadTags(0);

var tagRenameRequester = {
  asyncGetURLResponse: function(resp) {
    const user = globals.userManager.getDefaultUser();
    const cb = function(a) {
      observerService.notifyObservers(null,"ext:delicious:tagxmlForDefaultUser",user.tags.asXml(true));
    };
    user.tags.reload(cb, user.getUsername(), user.getPassword());
  },
  asyncGetURLError: function(error) {
    alert("Del.icio.us extension failed renaming the tag.");
    return;
  }
};

const sidebarObserver = {
  observe: function(subject,topic,data){
    switch (topic) {
      case "ext:delicious:sidebarLoaded":
        observerService.notifyObservers(null,"ext:delicious:defaultUserChanged",globals.userManager.getDefaultUser().getUsername());
        observerService.notifyObservers(null,"ext:delicious:tagxmlForDefaultUser",globals.userManager.getDefaultUser().tags.asXml(true));
        break;
      case "ext:delicious:sidebarReloadTagsRequest":
        const user = globals.userManager.getDefaultUser();
        const cb = function(a) {
          observerService.notifyObservers(null,"ext:delicious:tagxmlForDefaultUser",user.tags.asXml(true));
        };
        user.tags.reload(cb, user.getUsername(), user.getPassword());
        break;
      case "ext:delicious:sidebarRenameTagRequest":
        var m = data.match(/^from:(.*)&&__to:(.*)/);
        if (m) {
          globals.deliciousService.renameTag(m[1], m[2], tagRenameRequester);
        }
        break;
    }
  }
};

observerService.addObserver(sidebarObserver,"ext:delicious:sidebarLoaded",false);
observerService.addObserver(sidebarObserver,"ext:delicious:sidebarReloadTagsRequest",false);
observerService.addObserver(sidebarObserver,"ext:delicious:sidebarRenameTagRequest",false);

//
// Gets the target URL either from RSS reader headers or from the active document.
//
function getUrl()
{
    var url = "";
    var expandedContentBaseBox = document.getElementById("expandedcontent-baseBox");
    if (expandedContentBaseBox) {
        url = document.getAnonymousElementByAttribute(expandedContentBaseBox,"anonid","headerValue").value;
    }
    if (!url) {
        url = window._content.document.location.href;
    }
    return url;
}

//
// Post current page
//
function postCurrentPage()
{
    var url = encodeURIComponent(getUrl());
    var description = encodeURIComponent(window._content.document.title);

    var focusedWindow = document.commandDispatcher.focusedWindow;
    var extended = encodeURIComponent(focusedWindow.getSelection());

    window.openDialog("chrome://delicious/content/add.xul", "", "chrome, resizable", {"url":url, "description":description, "extended":extended, "globals":globals});
}

//
// Post clicked link
//
function postClickedLink()
{
    var url;
    var description;
    var extended = encodeURIComponent(window._content.getSelection());

    try
    {
        url = encodeURIComponent(gContextMenu.linkURL());
        description = encodeURIComponent(gContextMenu.linkText());

        window.openDialog("chrome://delicious/content/add.xul", "", "chrome, resizable", {"url":url, "description":description, "extended":extended, "globals":globals});
    }
    catch(e)
    {
        // TODO show error that should be shown
        alert("You have to right-click on a link...");
    }
}

//
// Finds the current page in del.icio.us
//
function findCurrentPage()
{
    var url = getUrl();

    globals.deliciousService.findURL(url);
}

//
// Finds the clicked link in del.icio.us
//
function findClickedLink()
{
    var clickedUrl;

    try
    {
        clickedUrl = gContextMenu.linkURL();
        globals.deliciousService.findURL(clickedUrl);
    }
    catch(e)
    {
        // TODO show error that should be shown
        alert("You have to right-click on a link...");
    }
}

